module OttomanORM
    class Model
        # representation methods
        include OttomanORM::Representation

        # internal references
        attr_accessor :_id

        # attributes list
        @@_attributes = []

        # Initializes a new model with the given +params+.
        #
        #   class Person
        #       include OttomanORM::Model
        #       attr_accessor :name, :age
        #   end
        #
        #   person = Person.new(name: 'bob', age: '18')
        #   person.name # => "bob"
        #   person.age  # => 18
        def initialize(params={})
            params.each do |attr, value|
                self.public_send("#{attr}=", value)
            end if params

            super()
        end

        # attributes
        def self.attribute *names
            names.each { |name| @@_attributes << name }
            class_eval { attr_accessor *names }
        end
        class << self; alias :attributes :attribute; end

        def attributes
            @@_attributes
        end

        # object creation
        def self.create data
            self.new(data).save
        end

        # Indicates if the model is persisted. Default is +false+.
        #
        #   class Person
        #       include OttomanORM::Model
        #       attr_accessor :id, :name
        #   end
        #
        #   person = Person.new(id: 1, name: 'bob')
        #   person.persisted? # => false
        def persisted?
            !new_record?
        end

        def new_record?
            @_id.blank?
        end

        def id
            @_id
        end

        # validations
        def run_validations
            true
        end

        # save record
        def save
            return false unless run_validations
            create_or_update
        end

        def create_or_update
            new_record? ? create_record : update_record
        end

        def create_record
            @_id = OttomanORM.client.create self.class.name, self.to_json
            self
        end

        def update_record
            OttomanORM.client.update @_id, self.to_hash
            self
        end

        def self.fetch name, id
            # r = []
            # OttomanORM.client.get(id).each_pair do |key, value|
            #     if v[0].is_a?(Hash)
            #         instance = new(value[0].extract!(*@@_attributes))
            #         instance._id = key
            #         r << instance
            #     end
            # end
            # r.empty? ? nil : r.length == 1 ? r[0] : r
            OttomanORM.client.get name, id
        end

        def self.fetch_by_field name, field, value
            OttomanORM.client.get_by_field name, field, value
        end

        # update record
        def update_attribute attribute, value
            public_send("#{attribute}=", value)
            save
        end

        def update_attributes attributes
            return if attributes.blank?
            attributes.stringify_keys.each_pair do |attribute, value|
                public_send("#{attribute}=", value)
            end
            save
        end

        # delete record
        def delete
            OttomanORM.client.delete(@_id) unless new_record?
            freeze
        end
    end
end
